splitApply = function(ds, splitVar, applyVar, fun){
  ###
  # Applies a function 'fun' to a column vector named 'applyVar' to a
  # dataframe 'ds' by groups defined in the column vector named 'splitVar'.
  # Similar to the tapply function, but using ddply
  a = ddply(ds, splitVar, function(x){fun(x[,c(applyVar)])})
  names(a)[2] = applyVar
  a
}


metrics.readDatasets.withNA = function(paths,names){
  ###
  # Creates a list of list with R dataframes and labels read from a list of
  # file paths ('paths') and a list of characters from the 'names' argument
  l = list()
  for (i in 1:length(paths)) {
    ds = read.csv(paths[i])
    l[[i]] = list(ds, names[i])
  }
  l
}

metrics.readDatasets = function(paths,names){
  ###
  # Creates a list of list with R dataframes and labels read from a list of
  # file paths ('paths') and a list of characters from the 'names' argument
  # and removes rows with at least one NA value.
  l = list()
  for (i in 1:length(paths)) {
    ds = read.csv(paths[i])
    toDelete = which(apply(is.na(ds),1,any))
    if (length(toDelete) > 0) {
      ds = ds[-toDelete,]
    }
    l[[i]] = list(ds, names[i])
  }
  l
}

splitCharCol = function(a,delim){
###
# Splits the string 'a' by 'delim' and creates a vector with the unique tokens
  apply(do.call('rbind', strsplit(as.character(a),delim,fixed=TRUE)), 1, unique)
}

overviewDataset = function(pfba,fmin,solds,fminbreaks,biomassName,productName,substrateValue){
### Generates a dataset with the most important metrics for design strategies
# Assuming s solutions and n reactions:
# 'pfba' is a dataframe (s by n) with s flux distributions
# 'fmin' is a dataframe (s by fminbreaks) with the minimum production flux
# at a certain minimum biomass demand.
# 'solds' is a dataframe with s rows and solution knockouts on the columns
# 'biomassName' - Biomass reaction ID; 'productName' - Product reaction ID;
# 'substrateValue' - Substrate uptake rate
  fmind = sapply(fminbreaks, function(i){fmin[,i]})
  fmaxp = pfba[,c(productName)]
  maxbio = pfba[,c(biomassName)]
  bpcy = (fmaxp*maxbio)/substrateValue
  solsize = dim(solds)[2] - apply(solds, 1, function(x){sum(is.na(x))})
  r = as.data.frame(fmind)
  names(r) = sapply(fminbreaks,function(x){paste("Robustness",(x-1)*(100/(length(fmin)-1)),sep = "")})
  r$MaximumProduct = fmaxp
  r$MaximumBiomass = maxbio
  r$BPCY = bpcy
  r$SolutionSize = solsize
  r
}

splitApplyDataFrame = function(ds, splitVar, fun, fName){
  ### Same as split-apply but adds a column 'fName' to the dataframe 'ds'
  a = merged.data.frame(lapply(names(ds)[-match(splitVar,names(ds))], function(x){splitApply(ds, splitVar, x, fun)}))
  a$Fun = fName
  a
}

getClampedOverviewDatasets = function(list){
  ### Merges overview datasets and adds an Alg column to identify each part
  addAlg = function(x){
    f = x[[1]]
    f$Algorithm = x[[2]]
    f
  }
  do.call(rbind, lapply(list, addAlg))
}

getMetricsPerSizeDataset = function(overviewList){
  ### Summarizes maximum and minimum metric values by solution size
  splymin = function(ds){splitApplyDataFrame(ds, "SolutionSize", min, "Minimum")}
  splymax = function(ds){splitApplyDataFrame(ds, "SolutionSize", max, "Maximum")}
  lst = lapply(overviewList, function(x){
    cur = rbind(splymin(x[[1]]), splymax(x[[1]]))
    cur$Algorithm = x[[2]]
    cur
  })
  do.call(rbind, lst)
}

metrics.boxplot = function(fullds, rows, cols, text.slant=60, text.size=12){
  ### Displays a boxplot of an overview dataset
  tgraphds = melt(data = fullds, id.vars = c("Algorithm","SolutionSize"))
  ggplot(tgraphds, aes(x = Algorithm, y = value)) +
    geom_boxplot(aes(group = Algorithm, fill = Algorithm)) +
    facet_wrap(~variable,scales = "free_y", nrow = rows, ncol = cols) +
    theme(axis.text.x = element_text(angle = text.slant, hjust = 1, size = text.size))
}

metrics.size.scatter = function(fullds, rows, cols, text.slant=60, text.size=12){
### Displays scatter plots of an overview dataset's columns
  graphds = melt(fullds, id.vars = c("SolutionSize", "Fun","Algorithm"))
  plt = ggplot(graphds[graphds$Fun == "Maximum",], aes(x = SolutionSize, y = value))
  plt +
    geom_point(aes(group = Algorithm, colour = Algorithm)) +
    geom_line(aes(group = Algorithm, colour = Algorithm)) +
    facet_wrap(~variable,scales = "free_y", nrow = rows, ncol = cols) +
    theme(axis.text.x = element_text(angle = text.slant, hjust = 1, size = text.size))
}

merged.data.frame = function(lst){Reduce(function(...) merge(..., all=T), lst)}
